package sun.nio.cs;

import java.nio.charset.CoderResult;
import java.nio.CharBuffer;
import java.nio.ByteBuffer;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.Charset;

public class US_ASCII extends Charset implements HistoricallyNamedCharset
{
    public US_ASCII() {
        super("US-ASCII", StandardCharsets.aliases_US_ASCII);
    }
    
    @Override
    public String historicalName() {
        return "ASCII";
    }
    
    @Override
    public boolean contains(Charset charset) {
        return charset instanceof US_ASCII;
    }
    
    @Override
    public CharsetDecoder newDecoder() {
        return new US_ASCII.Decoder(this);
    }
    
    @Override
    public CharsetEncoder newEncoder() {
        return new US_ASCII.Encoder(this);
    }
    
    private static class Decoder extends CharsetDecoder implements ArrayDecoder
    {
        private char repl;
        
        private Decoder(Charset cs) {
            super(cs, 1.0f, 1.0f);
            this.repl = '�';
        }
        
        private CoderResult decodeArrayLoop(ByteBuffer byteBuffer, CharBuffer charBuffer) {
            byte[] array = byteBuffer.array();
            int n = byteBuffer.arrayOffset() + byteBuffer.position();
            int n2 = byteBuffer.arrayOffset() + byteBuffer.limit();
            assert n <= n2;
            int i = (n <= n2) ? n : n2;
            char[] array2 = charBuffer.array();
            int n3 = charBuffer.arrayOffset() + charBuffer.position();
            int n4 = charBuffer.arrayOffset() + charBuffer.limit();
            assert n3 <= n4;
            int n5 = (n3 <= n4) ? n3 : n4;
            try {
                while (i < n2) {
                    int n6 = array[i];
                    if (n6 >= 0) {
                        if (n5 >= n4) {
                            return CoderResult.OVERFLOW;
                        }
                        array2[n5++] = (char)n6;
                        ++i;
                    }
                    else {
                        return CoderResult.malformedForLength(1);
                    }
                }
                return CoderResult.UNDERFLOW;
            }
            finally {
                byteBuffer.position(i - byteBuffer.arrayOffset());
                charBuffer.position(n5 - charBuffer.arrayOffset());
            }
        }
        
        private CoderResult decodeBufferLoop(ByteBuffer byteBuffer, CharBuffer charBuffer) {
            int position = byteBuffer.position();
            try {
                while (byteBuffer.hasRemaining()) {
                    int value = byteBuffer.get();
                    if (value >= 0) {
                        if (!charBuffer.hasRemaining()) {
                            return CoderResult.OVERFLOW;
                        }
                        charBuffer.put((char)value);
                        ++position;
                    }
                    else {
                        return CoderResult.malformedForLength(1);
                    }
                }
                return CoderResult.UNDERFLOW;
            }
            finally {
                byteBuffer.position(position);
            }
        }
        
        @Override
        protected CoderResult decodeLoop(ByteBuffer byteBuffer, CharBuffer charBuffer) {
            if (byteBuffer.hasArray() && charBuffer.hasArray()) {
                return this.decodeArrayLoop(byteBuffer, charBuffer);
            }
            return this.decodeBufferLoop(byteBuffer, charBuffer);
        }
        
        @Override
        protected void implReplaceWith(String s) {
            this.repl = s.charAt(0);
        }
        
        @Override
        public int decode(byte[] array, int n, int min, char[] array2) {
            int i = 0;
            min = Math.min(min, array2.length);
            while (i < min) {
                int n2 = array[n++];
                if (n2 >= 0) {
                    array2[i++] = (char)n2;
                }
                else {
                    array2[i++] = this.repl;
                }
            }
            return i;
        }
    }
    
    private static class Encoder extends CharsetEncoder implements ArrayEncoder
    {
        private final Surrogate.Parser sgp;
        private byte repl;
        
        private Encoder(Charset cs) {
            super(cs, 1.0f, 1.0f);
            this.sgp = new Surrogate.Parser();
            this.repl = 63;
        }
        
        @Override
        public boolean canEncode(char c) {
            return c < '\u0080';
        }
        
        @Override
        public boolean isLegalReplacement(byte[] repl) {
            return (repl.length == 1 && repl[0] >= 0) || super.isLegalReplacement(repl);
        }
        
        private CoderResult encodeArrayLoop(CharBuffer charBuffer, ByteBuffer byteBuffer) {
            char[] array = charBuffer.array();
            int n = charBuffer.arrayOffset() + charBuffer.position();
            int n2 = charBuffer.arrayOffset() + charBuffer.limit();
            assert n <= n2;
            int i = (n <= n2) ? n : n2;
            byte[] array2 = byteBuffer.array();
            int n3 = byteBuffer.arrayOffset() + byteBuffer.position();
            int n4 = byteBuffer.arrayOffset() + byteBuffer.limit();
            assert n3 <= n4;
            int n5 = (n3 <= n4) ? n3 : n4;
            try {
                while (i < n2) {
                    int n6 = array[i];
                    if (n6 < '\u0080') {
                        if (n5 >= n4) {
                            return CoderResult.OVERFLOW;
                        }
                        array2[n5] = (byte)n6;
                        ++i;
                        ++n5;
                    }
                    else {
                        if (this.sgp.parse((char)n6, array, i, n2) < 0) {
                            return this.sgp.error();
                        }
                        return this.sgp.unmappableResult();
                    }
                }
                return CoderResult.UNDERFLOW;
            }
            finally {
                charBuffer.position(i - charBuffer.arrayOffset());
                byteBuffer.position(n5 - byteBuffer.arrayOffset());
            }
        }
        
        private CoderResult encodeBufferLoop(CharBuffer charBuffer, ByteBuffer byteBuffer) {
            int position = charBuffer.position();
            try {
                while (charBuffer.hasRemaining()) {
                    int value = charBuffer.get();
                    if (value < '\u0080') {
                        if (!byteBuffer.hasRemaining()) {
                            return CoderResult.OVERFLOW;
                        }
                        byteBuffer.put((byte)value);
                        ++position;
                    }
                    else {
                        if (this.sgp.parse((char)value, charBuffer) < 0) {
                            return this.sgp.error();
                        }
                        return this.sgp.unmappableResult();
                    }
                }
                return CoderResult.UNDERFLOW;
            }
            finally {
                charBuffer.position(position);
            }
        }
        
        @Override
        protected CoderResult encodeLoop(CharBuffer charBuffer, ByteBuffer byteBuffer) {
            if (charBuffer.hasArray() && byteBuffer.hasArray()) {
                return this.encodeArrayLoop(charBuffer, byteBuffer);
            }
            return this.encodeBufferLoop(charBuffer, byteBuffer);
        }
        
        @Override
        protected void implReplaceWith(byte[] array) {
            this.repl = array[0];
        }
        
        @Override
        public int encode(char[] array, int i, int a, byte[] array2) {
            int n = 0;
            int n2 = i + Math.min(a, array2.length);
            while (i < n2) {
                int ch = array[i++];
                if (ch < '\u0080') {
                    array2[n++] = (byte)ch;
                }
                else {
                    if (Character.isHighSurrogate((char)ch) && i < n2 && Character.isLowSurrogate(array[i])) {
                        if (a > array2.length) {
                            ++n2;
                            --a;
                        }
                        ++i;
                    }
                    array2[n++] = this.repl;
                }
            }
            return n;
        }
    }
}
